package swtpl.marsrover.map;

import swtpl.marsrover.net.InitHandler;
import swtpl.marsrover.net.ObstacleEvent;
import swtpl.marsrover.net.ObstacleHandler;
import swtpl.marsrover.net.RunContext;

public class QuadTree implements InitHandler,ObstacleHandler {

/*	public static final int COMPLEX = -1;
	public static final int UNKNOWN = 0;
	public static final int PLAIN   = 1;
	public static final int BOULDER = 2;
	public static final int CRATER  = 3;
	public static final int MARTIAN = 4;*/
	
	
	public class Node {
		/** Coordinates of center */
		MapStateEnum type;
		ObstacleEvent obstacle;
		Node nw,ne,sw,se;
		
		public Node(MapStateEnum type) {
			this.type = type;
		}
	}
	
	/** The size of the topmost Node in mm (size of deepest leaf). */
	int rootSize;
	Node root;	

	private int resolution = 20;
	
	private void insertCircle(Node n, int nx, int ny, int nsize, 
			int x, int y, int radius, ObstacleEvent obstacle) {
		if (n.type.value() >= obstacle.getSort().value()) {
			/* This node type takes precedence over the inserted one. */
			return;
		}

		int ediff = x - nx - nsize;
		int wdiff = x - nx + nsize;
		int ndiff = y - ny - nsize;
		int sdiff = y - ny + nsize;
		/* Test whether bounding box of circle is outside node */
		if (ndiff > radius || ediff > radius
			|| wdiff < -radius || sdiff < -radius)
			return;
		/* Test whether circle is completely outside node */
		if (ediff > 0) {
			/* Circle is on east */
			if (ndiff >= 0) {
				if (ndiff*ndiff + ediff*ediff >= radius*radius)
					return;
			} else if (sdiff < 0) {
				if (sdiff*sdiff + ediff*ediff >= radius*radius)
					return;
			}
		}
		if (wdiff < 0) {
			/* Circle is on west */
			if (ndiff >= 0) {
				if (ndiff*ndiff + wdiff*wdiff >= radius*radius)
					return;
			} else if (sdiff < 0) {
				if (sdiff*sdiff + wdiff*wdiff >= radius*radius)
					return;
			}
		}
		/* Check whether node completely inside circle or 
		 * resolution depth reached */
		if (((long)ndiff * ndiff + (long)wdiff * wdiff <= (long)radius*radius
			&& (long)sdiff * sdiff + (long)wdiff * wdiff <= (long)radius*radius
			&& (long)ndiff * ndiff + (long)ediff * ediff <= (long)radius*radius
			&& (long)sdiff * sdiff + (long)ediff * ediff <= (long)radius*radius)
				|| nsize < resolution) {
				n.type = obstacle.getSort();
				n.obstacle = obstacle;
				lastObstacle = obstacle;
				lastX = nx;
				lastY = ny;
				lastSize = nsize;
				return;			
			}
		
		if (n.type != MapStateEnum.COMPLEX) {
			n.nw = new Node(n.type);
			n.ne = new Node(n.type);
			n.sw = new Node(n.type);
			n.se = new Node(n.type);
			n.type = MapStateEnum.COMPLEX;
		}
		nsize /= 2;
		insertCircle(n.nw, nx - nsize, ny + nsize, nsize, x, y, radius, obstacle);
		insertCircle(n.ne, nx + nsize, ny + nsize, nsize, x, y, radius, obstacle);
		insertCircle(n.sw, nx - nsize, ny - nsize, nsize, x, y, radius, obstacle);
		insertCircle(n.se, nx + nsize, ny - nsize, nsize, x, y, radius, obstacle);
		if (n.nw.type != MapStateEnum.COMPLEX && n.nw.type == n.ne.type && n.nw.type == n.sw.type && n.nw.type == n.se.type) {
			/* merge nodes again */
			n.type = n.nw.type;
			n.obstacle = n.nw.obstacle;
			n.nw = n.ne = n.sw = n.se = null;
		}
	}
	
	public void insertCircle(int x, int y, int radius, ObstacleEvent obstacle) {
		radius += 10;
		if (obstacle.getSort() == MapStateEnum.BOULDER)
			radius += 500;
		insertCircle(root, 0, 0, rootSize, x, y, radius, obstacle);
	}
	
	private void insertRectangle(Node n, int nx, int ny, int nsize, 
			int x1, int y1, int x2, int y2, MapStateEnum type) {
		if (n.type.value() >= type.value()) {
			/* This node type takes precedence over the inserted one. */
			return;
		}
		/* Rectangle outside node ? */
		if (x2 <= nx - nsize || x1 >= nx + nsize
			|| y2 <= ny - nsize || y1 >= ny + nsize)
			return;
		/* Check whether node completely inside rectangle or 
		 * resolution depth reached */
		if ((x1 <= nx - nsize && x2 >= nx + nsize
			&& y1 <= ny - nsize && y2 >= ny + nsize)
			|| nsize < resolution) {
			n.type = type;
			return;			
		}
		
		if (n.type != MapStateEnum.COMPLEX) {
			n.nw = new Node(n.type);
			n.ne = new Node(n.type);
			n.sw = new Node(n.type);
			n.se = new Node(n.type);
			n.type = MapStateEnum.COMPLEX;
		}
		nsize /= 2;
		insertRectangle(n.nw, nx - nsize, ny + nsize, nsize, x1, y1, x2, y2, type);
		insertRectangle(n.ne, nx + nsize, ny + nsize, nsize, x1, y1, x2, y2, type);
		insertRectangle(n.sw, nx - nsize, ny - nsize, nsize, x1, y1, x2, y2, type);
		insertRectangle(n.se, nx + nsize, ny - nsize, nsize, x1, y1, x2, y2, type);
		if (n.nw.type != MapStateEnum.COMPLEX && n.nw.type == n.ne.type && n.nw.type == n.sw.type && n.nw.type == n.se.type) {
			/* merge nodes again */
			n.type = n.nw.type;
			n.nw = n.ne = n.sw = n.se = null;
		}
	}

	public void insertRectangle(int x1, int y1, int x2, int y2, MapStateEnum type) {
		insertRectangle(root, 0, 0, rootSize, x1, y1, x2, y2, type);
	}
	
	
	public QuadTree() {
	}
	
	public void initMap(int xsize, int ysize) {
		int size = resolution;
	    while (size < xsize/2 || size < ysize/2)
	    	size*=2;
		root = new Node(MapStateEnum.UNKNOWN);
		rootSize = size;
		insertRectangle(-size, -size, +size, -ysize/2, MapStateEnum.BOULDER);
		insertRectangle(-size, ysize/2, +size, +size, MapStateEnum.BOULDER);
		insertRectangle(-size, -size, -xsize/2, +size, MapStateEnum.BOULDER);
		insertRectangle(xsize/2, -size, +size, +size, MapStateEnum.BOULDER);
	}
	
	public void dump(Node n, int nx, int ny, int size, String indent) {
		System.err.println(indent+ "("+nx+","+ny+":"+size+"): "+n.type);
		if (n.type == MapStateEnum.COMPLEX) {
			size/=2; indent += "  ";
			dump(n.nw, nx-size, ny +size, size, indent);
			dump(n.ne, nx+size, ny +size, size, indent);
			dump(n.sw, nx-size, ny -size, size, indent);
			dump(n.se, nx+size, ny -size, size, indent);
		}
	}

	@Override
	public void initialized(RunContext rc) {
		initMap(rc.getCardSizeX(),rc.getCardSizeY());
	}

	@Override
	public void recognized(ObstacleEvent oe) {
		insertCircle(oe.getX(),oe.getY(),oe.getRadius(),oe);
	}
	
	
	private ObstacleEvent lastObstacle;
	private int lastX;
	private int lastY;
	private int lastSize;
	private ObstacleEvent searchField(Node n, int nx, int ny, int nsize, int x, int y) {
		if (n.type == MapStateEnum.COMPLEX) {
			nsize /= 2;
			if (x < nx) {
				if (y < ny) {
					return searchField(n.sw, nx-nsize, ny-nsize, nsize, x, y);
				} else  {
					return searchField(n.nw, nx-nsize, ny+nsize, nsize, x, y);
				} 
			} else {
				if (y < ny) {
					return searchField(n.se, nx+nsize, ny-nsize, nsize, x, y);
				} else  {
					return searchField(n.ne, nx+nsize, ny+nsize, nsize, x, y);
				} 
			}
		}
		lastObstacle = n.obstacle;
		lastX = nx;
		lastY = ny;
		lastSize = nsize;
		return lastObstacle;
	}
	
	public ObstacleEvent getField(int x, int y) {
		if (x >= lastX - lastSize && x <= lastX + lastSize
				&& y >= lastY - lastSize && y <= lastY + lastSize)
			return lastObstacle;
		return searchField(root, 0, 0, rootSize, x, y);
	}
}
