package swtpl.marsrover.map;

import javax.swing.SwingUtilities;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import swtpl.marsrover.map.QuadTree.Node;
import swtpl.marsrover.net.ObstacleEvent;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.util.ArrayList;

public class QuadTreeTest extends JPanel implements Runnable{
	
	private int mapHeight;
	private int mapWidth;
	private int baseX;
	private int baseY;
	private int roverX;
    private int roverY;
    private int roverVisionMin;
    private int roverVisionWidth;
    private int roverVisionHeight;
    
    private static final Color roverVisionColor = new Color(255, 255, 255);
    private static final Color baseColor = new Color(0, 250, 0);
    private static final Color roverColor = new Color(83, 25, 147);
    private static final Color craterColor = new Color(147, 82, 0);
    private static final Color boulderColor = new Color(120, 120, 120);
    private static final Color martianColor = new Color(255, 37, 0);
    
    private QuadTree qtree;
    
    public static int SIZE = 50000;
	
	//test it
    public static void main(String[] args) {
    	QuadTreeTest clientGui = new QuadTreeTest(SIZE, SIZE, -100, -100, 40, 20);
    }
    
    //all parameters are rover coordinates, not java coordinates
    public QuadTreeTest(int mapHeight, int mapWidth, int roverX, int roverY, int roverVisionMax, int roverVisionMin){
    	this.mapHeight = mapHeight;
    	this.mapWidth = mapWidth;
    	this.roverX = toJavaX(roverX);
    	this.roverY = toJavaY(roverY);
    	this.baseX = this.mapHeight/2;
    	this.baseY = this.mapWidth/2;
        this.roverVisionMin = roverVisionMin;
        this.roverVisionWidth = roverVisionMin + roverVisionMax;
        this.roverVisionHeight = 2*roverVisionMin;

		qtree = new QuadTree();
		qtree.initMap(400000, 250000);
		for (int i = 0; i < 1000; i++) {
			int x = (int) (Math.random()*500000) - 250000;
			int y = (int) (Math.random()*200000) - 100000;
			int radius = 3000;
			qtree.insertCircle(x, y, radius, 
					new ObstacleEvent(0, MapStateEnum.CRATER, x, y, radius));
		}

		//init gui
    	SwingUtilities.invokeLater(this);

    }

    //convert rover coordinates to java coordinates 
    private int toJavaX(int roverX){
    	return roverX + (this.mapHeight/2);
    }
    private int toJavaY(int roverY){
    	return this.mapHeight/2 - roverY;
    }
    
	public void run() {
        JFrame f = new JFrame("Map " + this.mapHeight + "x" + this.mapWidth);
        f.setPreferredSize(new Dimension(1000,800));
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        f.add(new JScrollPane(this));
        f.pack();
        f.setVisible(true);
    }
       
    public Dimension getPreferredSize() {
        return new Dimension(this.mapHeight, this.mapWidth);
    }

    public void draw(Graphics g, Node n, int nx, int ny, int size) {
		double factor = (double)SIZE / qtree.rootSize/2;
		int x = (int) ((nx + qtree.rootSize) * factor);
		int y = (int) ((-ny + qtree.rootSize) * factor);
		int s = (int) (size* factor);
		if (!g.hitClip(x-s, y-s, 2*s, 2*s))
			return;
        if (s == 0) {
        	g.setColor(Color.WHITE);
        	g.drawLine(x,y,x,y);
        	return;
        }
		
		if (n.type == MapStateEnum.COMPLEX) {
			size/=2;
			draw(g,n.nw, nx-size, ny +size, size);
			draw(g,n.ne, nx+size, ny +size, size);
			draw(g,n.sw, nx-size, ny -size, size);
			draw(g,n.se, nx+size, ny -size, size);
		} else {
			if (n.type == MapStateEnum.BOULDER) {
		        g.setColor(QuadTreeTest.boulderColor);
			} else if (n.type == MapStateEnum.CRATER) {
			    g.setColor(QuadTreeTest.craterColor);
			} else {
				g.setColor(QuadTreeTest.baseColor);
			}
	        g.fillRect(x-s,y-s,x+s,y+s);
	        g.setColor(Color.WHITE);
	        g.drawRect(x-s,y-s,x+s,y+s);
		}
	}

    //does the actual painting
    public void paintComponent(Graphics g) {
        super.paintComponent(g);       
        
		draw(g, qtree.root, 0, 0, qtree.rootSize);
    }
}
