package swtpl.marsrover.planner;

import java.awt.Color;
import java.util.ArrayList;

import swtpl.marsrover.gui.PaintObserver;
import swtpl.marsrover.gui.Painter;
import utils.Metrics;

public class ObstaclePath implements Painter {
	static public class Obstacle {
		public int x,y,radius;
		public boolean evadeLeft;
		int apexX, apexY, apexDir;
		int outDir, outRadius;
		int outSpeed;
		int outCenterX, outCenterY;
		int inCenterX, inCenterY;

		public Obstacle(int x, int y, int radius, boolean evadeLeft) {
			this.x = x;
			this.y = y;
			this.radius = radius;
			this.evadeLeft = evadeLeft;
		}
	}
	
	public ArrayList<Obstacle> waypoints;
	
	public ObstaclePath() {
		waypoints = new ArrayList<Obstacle>();
	}
	
	public synchronized void add(int pos, Obstacle elem) {
		for (Obstacle wp : waypoints) {
			if (wp.x == elem.x && wp.y == elem.y)
				return;
		}
		waypoints.add(pos, elem);
	}

	public void paintComponent(PaintObserver po) {
		synchronized (this) {
			int lastX = 0;
			int lastY = 0;
			for (Obstacle obs : waypoints) {
				po.setColor(Color.red);
				po.drawCircle(obs.x, obs.y, obs.radius);
				if (lastX != 0 || lastY != 0)
					po.drawLine(lastX, lastY, obs.apexX, obs.apexY);
				po.setColor(Color.magenta);
				po.drawLine(obs.apexX, obs.apexY, 
						obs.apexX+Metrics.scaledCos(obs.apexDir, 20000),
						obs.apexY+Metrics.scaledSin(obs.apexDir, 20000));
				lastX = obs.apexX;
				lastY = obs.apexY;
			}
		}
	}
}	
