package swtpl.marsrover.planner;

import swtpl.marsrover.net.Connection;
import utils.Metrics;
import swtpl.marsrover.gui.PaintObserver;

public class PathPlanner extends Planner {
	Path path;
	static final int REACHED_DIST = 2000;
	
	public PathPlanner(Connection con, PaintObserver po) {
		super(con,po);
		path = new Path();
	}
	
	public boolean reached(Path.Waypoint p) {
		boolean r = Metrics.inCircle(REACHED_DIST, p.x-rover.getX(), p.y-rover.getY());
		if(r) utils.Debug.println("reached waypoint");
		return r;
	}
	
	public void run() {
		path.waypoints.push(new Path.Waypoint(0,0));
		path.waypoints.push(new Path.Waypoint(-40000, 40000));
		path.waypoints.push(new Path.Waypoint(-40000,-40000));
		path.waypoints.push(new Path.Waypoint( 40000,-40000));	
		path.waypoints.push(new Path.Waypoint( 40000, 40000));
		
		synchronized(rover) {
			try {
				while (!rover.positionKnown) {
					rover.wait();
				}
			} catch (InterruptedException ex) {
				System.err.println("Can't wait for position:"+ex);
				return;
			}
		}

		while(true) {
			try {
				Thread.sleep(200);
			} catch (InterruptedException ex) {
				System.err.println("Can't wait for position:"+ex);
				return;
			}

			Path.Waypoint p=null;
			
			if(!path.waypoints.isEmpty()) {
				p = path.waypoints.getFirst();
				rover.setDesiredTarget(p.x, p.y);
			} else {
				rover.setDesiredStearing(-1, 0);
			}
			
			if(p!=null && reached(p))
				path.waypoints.pop();
		}
	}

}
