package swtpl.marsrover.planner;

import java.awt.Point;
import java.util.ArrayList;
import java.util.Stack;

import utils.Metrics;
import swtpl.marsrover.gui.PaintObserver;
import swtpl.marsrover.map.QuadTree;
import swtpl.marsrover.net.CleanupBase;
import swtpl.marsrover.net.Connection;
import swtpl.marsrover.net.InitHandler;
import swtpl.marsrover.net.ObstacleEvent;
import swtpl.marsrover.net.RunContext;
import swtpl.marsrover.net.VehicleEvent;
import swtpl.marsrover.net.VehicleHandler;
import swtpl.marsrover.net.VehicleEvent.VehicleControlAcceleration;
import swtpl.marsrover.net.VehicleEvent.VehicleControlTurning;

abstract public class Planner implements InitHandler, Runnable, CleanupBase {
	Rover rover;
	QuadTree quadtree;
	Thread thread;
	ArrayList<GoalHandler> goalHandlers = null;
    RunContext rc;
    PaintObserver po;
	
	public void registerGoalHandler(GoalHandler gh) {
		synchronized( this ) {
			if (goalHandlers == null)
				goalHandlers = new ArrayList<GoalHandler>();
			goalHandlers.add(gh);
		}
	}

	public Planner(Connection con, PaintObserver po) {
		this.po = po;
		rover = new Rover(con);
		quadtree = new QuadTree();
		con.registerInitHandler(this);
		con.registerInitHandler(quadtree);
		con.registerObstacleHandler(quadtree);
		con.registerCleanup(this);
	}
	
	public void initialized(RunContext rc) {
        this.rc = rc;
		thread = new Thread(this);
		thread.start();
	}

	public void doCleanup() {
		rover.doCleanup();
    }
	
	public abstract void run();
}
