package swtpl.marsrover.planner;

import swtpl.marsrover.gui.ClientGui;
import swtpl.marsrover.gui.PaintObserver;
import swtpl.marsrover.gui.Painter;
import swtpl.marsrover.net.RunContext;
import swtpl.marsrover.net.VehicleEvent.VehicleControlAcceleration;
import swtpl.marsrover.net.VehicleEvent.VehicleControlTurning;
import utils.*;
import java.awt.*;
import java.util.LinkedList;

public class Predicter implements Painter {
	final static int STEP = 10;
	final static LinkedList<Point> plist = new LinkedList<Point>();
	
	private final int accel;
	private final int brake;
	private final double drag;

	private double x;
	private double y;
	private double speed;
	private double d;
	private double deltaT;
	

	public Predicter(PaintObserver po, double accel, double brake, double drag,double x, double y, double speed, double d, int deltaT) {
		System.err.println("Register as painter");
		po.addPainter(this);
		this.accel = (int) accel;
		this.brake = (int) brake;
		this.drag = drag;
		this.x = x;
		this.y = y;
		this.speed = speed;
		this.d = d;
		this.deltaT = (double) deltaT / 1000.0;
	}
	
	public int getX() {
		return (int) x;
	}
	
	public int getY() {
		return (int) y;
	}
	
	public int getDirection() {
		return (int) d;
	}
	
	public void setSpeed(int speed) {
		this.speed = (double) speed;
	}
	
	private void predictPosition(VehicleControlAcceleration brkacc,VehicleControlTurning turn) {
		double a;
		int angAdd;
		switch (brkacc) {
		case ACCELERATING:
			a = accel;
			break;
		case BRAKING:
			a = brake;
			break;
		case ROLLING:
		default:
			a = 0;
			break;
		}
		switch (turn) {
		case HARDRIGHT:
			angAdd = -RunContext.getCurrentContext().getTenthDegreePerSecsHardTurn();
			break;
		case RIGHT:
			angAdd = -RunContext.getCurrentContext().getTenthDegreePerSecsTurn();
			break;
		case LEFT:
			angAdd = RunContext.getCurrentContext().getTenthDegreePerSecsTurn();
			break;
		case HARDLEFT:
			angAdd = RunContext.getCurrentContext().getTenthDegreePerSecsHardTurn();
			break;
		case STRAIGHT:
		default:
			angAdd = 0;
			break;
		}
		speed += (int) (deltaT * a - drag * deltaT * speed*speed);
		double dpi18 = d*Math.PI/1800 ;
		if (speed < 0)	speed = 0;
		x += Math.cos(dpi18) * speed * deltaT;
		y += Math.sin(dpi18) * speed * deltaT;
		d += deltaT * angAdd;
		if (d > 1800) d -= 3600;
		if (d < -1800) d += 3600;
	}

	private int angleDist() {
		return Metrics.normalizedAngle(Metrics.angleTowards((int) x, (int) y,0,0) - (int) this.d);
	}
	
	public void simulate() {
		double mind = RunContext.getCurrentContext().getTenthDegreePerSecsTurn() * deltaT * 2/3;
		double minD = RunContext.getCurrentContext().getTenthDegreePerSecsHardTurn() * deltaT * 2/3;
		int dDist = this.angleDist();
		VehicleControlTurning turn = VehicleControlTurning.HARDLEFT;
		if (dDist < 0) turn =  VehicleControlTurning.HARDRIGHT;
		dDist = Math.abs(dDist);
		
		while (dDist > mind) {
			System.err.println("Comp. Point");
			if (dDist < minD) {
				if (turn == VehicleControlTurning.HARDLEFT) {
					turn = VehicleControlTurning.LEFT;
				} else if (turn == VehicleControlTurning.HARDRIGHT) {
					turn = VehicleControlTurning.RIGHT;
				}
			}
			plist.add(new Point((int) this.x,(int) this.y));
			this.predictPosition( VehicleControlAcceleration.ACCELERATING, turn );
			dDist = Math.abs(this.angleDist());
		}
		
	}

	@Override
	public void paintComponent(PaintObserver po) {
		po.setColor(Color.magenta);
		System.err.println("Zeichnen");
		for (Point d : this.plist) {
			po.fillCircle(d.x, d.y, 400);
		}		
	}
	
}
