package swtpl.marsrover.planner;

import java.awt.Point;
import java.util.Stack;

import swtpl.marsrover.gui.PaintObserver;
import swtpl.marsrover.net.Connection;

public class RasterPlanner extends Planner {

	private final Stack<Point> goals = new Stack<Point>();
	private final Point home = new Point(0,0);
	
	public RasterPlanner(Connection con, PaintObserver po) {
		super(con,po);
		goals.add(home);
	}

	private static int normalizeAngle(int angle) {
		if (angle > 1800)
			return angle - 3600;
		if (angle < -1800)
			return angle + 3600;
		return angle;
	}

	private void adujstAngle() {
		Point g = goals.peek();
		double destDir = Math.atan2(g.y - rover.getY(), g.x - rover.getX());
		int delta = normalizeAngle((int) (destDir* 3600 / (2*Math.PI)) - this.rover.getDir());
		System.err.println("Angle Delta: " + delta);
		int wantTurn;
		if (Math.abs(delta) < rc.getTenthDegreePerSecsTurn()/4)
			wantTurn = 0;
		else if (Math.abs(delta) < 1*rc.getTenthDegreePerSecsHardTurn()/5)
			wantTurn = 1;
		else
			wantTurn = 2;
		if (delta < 0)
			wantTurn = -wantTurn;
		
		this.rover.setDesiredStearing(1, wantTurn);
		
	}
	
	@Override
	public void run() {
		// TODO Auto-generated method stub
		
		System.err.println("Running Planner");
		synchronized(rover) {
			try {
				while (!rover.positionKnown)
					rover.wait();
				rover.setDesiredStearing(1, 0);					
			} catch (InterruptedException ex) {
				System.err.println("Can't wait for position:"+ex);
				return;
			}
		}
		synchronized(rover) {
			try {
				while (!rover.accelKnown)
					rover.wait();
			} catch (InterruptedException ex) {
				System.err.println("Can't wait for acc:"+ex);
				return;
			}
		}
		Predicter pre = new Predicter(this.po,
				this.rover.getAccel(),rover.getBrake(),rover.getDrag(),
				rover.getX(),rover.getY(),rover.getSpeed(),rover.getDir(),500);		
		pre.simulate();
		while (true) {
			try {
				Thread.sleep(100);
			} catch (InterruptedException ex) {
				System.err.println("Can't wait for position:"+ex);
				return;
			}
			this.adujstAngle();
		}
	}

	public void doCleanup() {
		System.err.println("Reset, new round");
		super.doCleanup();
		goals.clear();
		goals.add(home);
	}
	
	
}
